@@ -14,14 +14,21 @@ This package provides a generic object editor widget.
 
 """
 
+from __future__ import annotations
 
-import PIL.Image
+from typing import TYPE_CHECKING
+
+try:
+    from PIL import Image as PILImage
+except ImportError:
+    PILImage = None
+from numpy.core.multiarray import ndarray
+
+from guidata.qthelpers import exec_dialog
 from guidata.widgets.arrayeditor import ArrayEditor
 from guidata.widgets.collectionseditor import CollectionsEditor
 from guidata.widgets.nsview import DataFrame, FakeObject, Series, is_known_type
 from guidata.widgets.texteditor import TextEditor
-from numpy.core.multiarray import ndarray
-from qtpy.QtCore import QObject
 
 try:
     from guidata.widgets.dataframeeditor import DataFrameEditor
@@ -29,6 +36,11 @@ except ImportError:
     DataFrameEditor = FakeObject()
 
 
+if TYPE_CHECKING:
+    import numpy as np
+    from qtpy import QtWidgets as QW
+
+
 def create_dialog(obj, title, parent=None):
     """Creates the editor dialog and returns a tuple (dialog, func) where func
     is the function to be called with the dialog instance as argument, after
@@ -39,20 +51,27 @@ def create_dialog(obj, title, parent=None):
     oedit to show eMZed related data)
     """
 
-    conv_func = lambda data: data
+    def conv_func(data):
+        """Conversion function"""
+        return data
+
     readonly = not is_known_type(obj)
     if isinstance(obj, ndarray):
         dialog = ArrayEditor(parent)
         if not dialog.setup_and_check(obj, title=title, readonly=readonly):
             return
-    elif isinstance(obj, PIL.Image.Image):
+    elif PILImage is not None and isinstance(obj, PILImage.Image):
         dialog = ArrayEditor(parent)
         import numpy as np
 
         data = np.array(obj)
         if not dialog.setup_and_check(data, title=title, readonly=readonly):
             return
-        conv_func = lambda data: PIL.Image.fromarray(data, mode=obj.mode)
+
+        def conv_func(data):  # pylint: disable=function-redefined
+            """Conversion function"""
+            return PILImage.fromarray(data, mode=obj.mode)
+
     elif isinstance(obj, (DataFrame, Series)) and DataFrame is not FakeObject:
         dialog = DataFrameEditor(parent)
         if not dialog.setup_and_check(obj):
@@ -74,20 +93,27 @@ def create_dialog(obj, title, parent=None):
     return dialog, end_func
 
 
-def oedit(obj, title=None, parent=None):
+def oedit(
+    obj: dict | list | tuple | str | np.ndarray,
+    title: str = None,
+    parent: QW.QWidget = None,
+) -> dict | list | tuple | str | np.ndarray:
     """Edit the object 'obj' in a GUI-based editor and return the edited copy
     (if Cancel is pressed, return None)
 
-    The object 'obj' is a container
+    Args:
+        obj (dict | list | tuple | str | np.ndarray): object to edit
+        title (str): dialog title
+        parent (QW.QWidget): parent widget
 
-    Supported container types:
-    dict, list, tuple, str/unicode or numpy.array
+    Returns:
+        dict | list | tuple | str | np.ndarray: edited object
     """
     title = "" if title is None else title
     result = create_dialog(obj, title, parent)
     if result is None:
         return
     dialog, end_func = result
-    if dialog.exec_():
+    if exec_dialog(dialog):
         return end_func(dialog)
     return None
